# SPDX-License-Identifier: MIT
# Copyright (C) 2021 Max Bachmann

from rapidfuzz import string_metric
import logging

def distance(s1, s2):
    logger = logging.getLogger(__name__)
    logger.warn(
        'This function is deprecated and will be removed in v2.0.0.\n'
        'Use string_metric.levenshtein(s1, s2) instead')
    return string_metric.levenshtein(s1, s2)

def normalized_distance(s1, s2, score_cutoff=0):
    logger = logging.getLogger(__name__)
    logger.warn(
        'This function is deprecated and will be removed in v2.0.0.\n'
        'Use string_metric.normalized_levenshtein(s1, s2, score_cutoff=%f) instead' % score_cutoff)
    return string_metric.normalized_levenshtein(s1, s2, score_cutoff=score_cutoff)

def weighted_distance(s1, s2, insert_cost=1, delete_cost=1, replace_cost=1):
    logger = logging.getLogger(__name__)
    logger.warn(
        'This function is deprecated and will be removed in v2.0.0.\n'
        'Use string_metric.normalized_levenshtein(s1, s2, insert_cost=%d, delete_cost=%d, replace_cost=%d) instead'
        % (insert_cost, delete_cost, replace_cost))
    return string_metric.levenshtein(s1, s2, insert_cost, delete_cost, replace_cost)

def normalized_weighted_distance(s1, s2, score_cutoff=0):
    logger = logging.getLogger(__name__)
    logger.warn(
        'This function is deprecated and will be removed in v2.0.0.\n'
        'Use string_metric.normalized_levenshtein(s1, s2, insert_cost=1, delete_cost=1, replace_cost=2, score_cutoff=%f) instead'
        %  score_cutoff)
    return string_metric.normalized_levenshtein(s1, s2, insert_cost=1, delete_cost=1, replace_cost=2, score_cutoff=score_cutoff)

def hamming(s1, s2):
    logger = logging.getLogger(__name__)
    logger.warn(
        'This function is deprecated and will be removed in v2.0.0.\n'
        'Use string_metric,hamming(s1, s2) instead')
    return string_metric,hamming(s1, s2)


from typing import Callable, Hashable, Sequence, Optional, Union, TypeVar, Tuple, List

_StringType = Sequence[Hashable]
S1 = TypeVar("S1")
S2 = TypeVar("S2")

def levenshtein(
    s1: S1, s2: S2, *,
    weights: Optional[Tuple[int, int, int]] = (1,1,1),
    processor: Optional[Callable[[Union[S1, S2]], _StringType]] = ...,
    max: Optional[int] = None) -> int: ...

def normalized_levenshtein(
    s1: S1, s2: S2, *,
    weights: Optional[Tuple[int, int, int]] = (1,1,1),
    processor: Optional[Callable[[Union[S1, S2]], _StringType]] = ...,
    score_cutoff: Optional[float] = 0) -> float: ...

def levenshtein_editops(
    s1: S1, s2: S2, *,
    processor: Optional[Callable[[Union[S1, S2]], _StringType]] = ...) -> List[Tuple[str, int, int]]: ...

def hamming(
    s1: S1, s2: S2, *,
    processor: Optional[Callable[[Union[S1, S2]], _StringType]] = ...,
    max: Optional[int] = None) -> int: ...

def normalized_hamming(
    s1: S1, s2: S2, *,
    processor: Optional[Callable[[Union[S1, S2]], _StringType]] = ...,
    score_cutoff: Optional[float] = 0) -> float: ...

def jaro_similarity(
    s1: S1, s2: S2, *,
    processor: Optional[Callable[[Union[S1, S2]], _StringType]] = ...,
    score_cutoff: Optional[float] = 0) -> float: ...

def jaro_winkler_similarity(
    s1: S1, s2: S2, *,
    prefix_weight: float = 0.1,
    processor: Optional[Callable[[Union[S1, S2]], _StringType]] = ...,
    score_cutoff: Optional[float] = 0) -> float: ...



from typing import Any, Collection, Mapping, Tuple, Callable, Hashable, Sequence, Iterable, Optional, Union, overload, TypeVar, List, Generator
from rapidfuzz.fuzz import WRatio, ratio

_StringType = Sequence[Hashable]
_AnyStringType = TypeVar("_AnyStringType", bound=_StringType)
S1 = TypeVar("S1")
S2 = TypeVar("S2")
ResultType = TypeVar("ResultType", int, float)

@overload
def extractOne(
    query: _StringType,
    choices: Iterable[_AnyStringType], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Optional[bool] = None,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> Tuple[_AnyStringType, ResultType, int]: ...

@overload
def extractOne(
    query: _StringType,
    choices: Mapping[Any, _AnyStringType], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Optional[bool] = None,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> Tuple[_AnyStringType, ResultType, Any]: ...

@overload
def extractOne(
    query: S1,
    choices: Iterable[S2], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Callable[[Union[S1, S2]], _StringType] = None,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> Tuple[S2, ResultType, int]: ...

@overload
def extractOne(
    query: S1,
    choices: Mapping[Any, S2], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Callable[[Union[S1, S2]], _StringType] = None,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> Tuple[S2, ResultType, Any]: ...

@overload
def extract(
    query: _StringType,
    choices: Collection[_AnyStringType], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Optional[bool] = None,
    limit: Optional[int] = ...,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> List[Tuple[_AnyStringType, ResultType, int]]: ...


@overload
def extract(
    query: _StringType,
    choices: Mapping[Any, _AnyStringType], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Optional[bool] = None,
    limit: Optional[int] = ...,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> List[Tuple[_AnyStringType, ResultType, Any]]: ...

@overload
def extract(
    query: S1,
    choices: Collection[S2], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Callable[[Union[S1, S2]], _StringType] = None,
    limit: Optional[int] = ...,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> List[Tuple[S2, ResultType, int]]: ...

@overload
def extract(
    query: S1,
    choices: Mapping[Any, S2], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Callable[[Union[S1, S2]], _StringType] = None,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> List[Tuple[S2, ResultType, Any]]: ...


@overload
def extract_iter(
    query: _StringType,
    choices: Iterable[_AnyStringType], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Optional[bool] = None,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> Generator[Tuple[_AnyStringType, ResultType, int], None, None]: ...

@overload
def extract_iter(
    query: _StringType,
    choices: Mapping[Any, _AnyStringType], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Optional[bool] = None,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> Generator[Tuple[_AnyStringType, ResultType, Any], None, None]: ...

@overload
def extract_iter(
    query: S1,
    choices: Iterable[S2], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Callable[[Union[S1, S2]], _StringType] = None,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> Generator[Tuple[S2, ResultType, int], None, None]: ...

@overload
def extract_iter(
    query: S1,
    choices: Mapping[Any, S2], *,
    scorer: Callable[..., ResultType] = WRatio,
    processor: Callable[[Union[S1, S2]], _StringType] = None,
    score_cutoff: Optional[ResultType] = None,
    **kwargs: Any
) -> Generator[Tuple[S2, ResultType, Any], None, None]: ...

try:
    import numpy as np
    @overload
    def cdist(
        queries: Iterable[S1],
        choices: Iterable[S2], *,
        scorer: Callable[..., ResultType] = ratio,
        processor: Optional[bool] = None,
        score_cutoff: Optional[ResultType] = None,
        dtype: Optional[np.dtype] = None,
        **kwargs: Any
    ) -> np.ndarray: ...
    
    @overload
    def cdist(
        queries: Iterable[S1],
        choices: Iterable[S2], *,
        scorer: Callable[..., ResultType] = ratio,
        processor: Callable[[Union[S1, S2]], _StringType] = None,
        score_cutoff: Optional[ResultType] = None,
        dtype: Optional[np.dtype] = None,
        **kwargs: Any
    ) -> np.ndarray: ...

except ImportError:
    pass



# SPDX-License-Identifier: MIT
# Copyright (C) 2021 Max Bachmann

from rapidfuzz.cpp_process import extract, extractOne, extract_iter

try:
    from rapidfuzz.cpp_process_cdist import cdist
except ImportError:
    def cdist(*args, **kwargs):
        raise NotImplementedError("implementation requires numpy to be installed")

